"use strict";

/**
 * A map of commands in-flight to debounce.
 *
 * @var {InflightStore}
 */
const inflight = new Map();

class Debouncer {
  /**
   * The store that resolves the commands.
   *
   * @var {Object}
   */

  /**
   * @inheritDoc
   */
  constructor(backend) {
    this.backend = backend;
  }
  /**
   * @inheritDoc
   */


  debounce(command, ...args) {
    const key = `${command}::${args.map(a => JSON.stringify(a)).join(':')}`;
    const debounced = inflight.get(key);

    if (debounced) {
      return debounced;
    }

    const promise = this.backend[command](...args).then(result => {
      // Remove from in-flight when it resolves and return the results.
      inflight.delete(key);
      return result;
    }).catch(err => {
      inflight.delete(key);
      throw err;
    }); // Add the promise to the in-flight map.

    inflight.set(key, promise);
    return promise;
  }

}

module.exports = Debouncer;